/*
 * Copyright (C) 2018 Open Source Robotics Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
*/

syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Twist
/// \brief Message with linear and angular velocities.

import "vector3d.proto";
import "header.proto";

message Twist
{
    /// \brief Optional header data.
    optional Header header          = 1;

    /// \brief Lnear velocity in 3d space.
    required Vector3d linear        = 2;

    /// \brief Angular velocity in 3d space.
    required Vector3d angular       = 3;
}