// Generated by the protocol buffer compiler.  DO NOT EDIT!
// source: joint.proto

#ifndef GOOGLE_PROTOBUF_INCLUDED_joint_2eproto
#define GOOGLE_PROTOBUF_INCLUDED_joint_2eproto

#include <limits>
#include <string>

#include <google/protobuf/port_def.inc>
#if PROTOBUF_VERSION < 3020000
#error This file was generated by a newer version of protoc which is
#error incompatible with your Protocol Buffer headers. Please update
#error your headers.
#endif
#if 3020000 < PROTOBUF_MIN_PROTOC_VERSION
#error This file was generated by an older version of protoc which is
#error incompatible with your Protocol Buffer headers. Please
#error regenerate this file with a newer version of protoc.
#endif

#include <google/protobuf/port_undef.inc>
#include <google/protobuf/io/coded_stream.h>
#include <google/protobuf/arena.h>
#include <google/protobuf/arenastring.h>
#include <google/protobuf/generated_message_util.h>
#include <google/protobuf/metadata_lite.h>
#include <google/protobuf/generated_message_reflection.h>
#include <google/protobuf/message.h>
#include <google/protobuf/repeated_field.h>  // IWYU pragma: export
#include <google/protobuf/extension_set.h>  // IWYU pragma: export
#include <google/protobuf/generated_enum_reflection.h>
#include <google/protobuf/unknown_field_set.h>
#include "axis.pb.h"
#include "pose.pb.h"
#include "sensor.pb.h"
#ifndef _MSC_VER
#pragma GCC system_header
#pragma GCC diagnostic ignored "-Wshadow"
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif
#include <boost/shared_ptr.hpp>
#include <gazebo/util/system.hh>
#include "gazebo/msgs/MsgFactory.hh"
// @@protoc_insertion_point(includes)
#include <google/protobuf/port_def.inc>
#define PROTOBUF_INTERNAL_EXPORT_joint_2eproto GZ_MSGS_VISIBLE
PROTOBUF_NAMESPACE_OPEN
namespace internal {
class AnyMetadata;
}  // namespace internal
PROTOBUF_NAMESPACE_CLOSE

// Internal implementation detail -- do not use these members.
struct GZ_MSGS_VISIBLE TableStruct_joint_2eproto {
  static const uint32_t offsets[];
};
GZ_MSGS_VISIBLE extern const ::PROTOBUF_NAMESPACE_ID::internal::DescriptorTable descriptor_table_joint_2eproto;
namespace gazebo {
namespace msgs {
class Joint;
struct JointDefaultTypeInternal;
GZ_MSGS_VISIBLE extern JointDefaultTypeInternal _Joint_default_instance_;
class Joint_Gearbox;
struct Joint_GearboxDefaultTypeInternal;
GZ_MSGS_VISIBLE extern Joint_GearboxDefaultTypeInternal _Joint_Gearbox_default_instance_;
class Joint_Screw;
struct Joint_ScrewDefaultTypeInternal;
GZ_MSGS_VISIBLE extern Joint_ScrewDefaultTypeInternal _Joint_Screw_default_instance_;
}  // namespace msgs
}  // namespace gazebo
PROTOBUF_NAMESPACE_OPEN
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::Joint* Arena::CreateMaybeMessage<::gazebo::msgs::Joint>(Arena*);
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::Joint_Gearbox* Arena::CreateMaybeMessage<::gazebo::msgs::Joint_Gearbox>(Arena*);
template<> GZ_MSGS_VISIBLE ::gazebo::msgs::Joint_Screw* Arena::CreateMaybeMessage<::gazebo::msgs::Joint_Screw>(Arena*);
PROTOBUF_NAMESPACE_CLOSE
namespace gazebo {
namespace msgs {

enum Joint_Type : int {
  Joint_Type_REVOLUTE = 1,
  Joint_Type_REVOLUTE2 = 2,
  Joint_Type_PRISMATIC = 3,
  Joint_Type_UNIVERSAL = 4,
  Joint_Type_BALL = 5,
  Joint_Type_SCREW = 6,
  Joint_Type_GEARBOX = 7,
  Joint_Type_FIXED = 8
};
GZ_MSGS_VISIBLE bool Joint_Type_IsValid(int value);
constexpr Joint_Type Joint_Type_Type_MIN = Joint_Type_REVOLUTE;
constexpr Joint_Type Joint_Type_Type_MAX = Joint_Type_FIXED;
constexpr int Joint_Type_Type_ARRAYSIZE = Joint_Type_Type_MAX + 1;

GZ_MSGS_VISIBLE const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor* Joint_Type_descriptor();
template<typename T>
inline const std::string& Joint_Type_Name(T enum_t_value) {
  static_assert(::std::is_same<T, Joint_Type>::value ||
    ::std::is_integral<T>::value,
    "Incorrect type passed to function Joint_Type_Name.");
  return ::PROTOBUF_NAMESPACE_ID::internal::NameOfEnum(
    Joint_Type_descriptor(), enum_t_value);
}
inline bool Joint_Type_Parse(
    ::PROTOBUF_NAMESPACE_ID::ConstStringParam name, Joint_Type* value) {
  return ::PROTOBUF_NAMESPACE_ID::internal::ParseNamedEnum<Joint_Type>(
    Joint_Type_descriptor(), name, value);
}
// ===================================================================

class GZ_MSGS_VISIBLE Joint_Gearbox final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.Joint.Gearbox) */ {
 public:
  inline Joint_Gearbox() : Joint_Gearbox(nullptr) {}
  ~Joint_Gearbox() override;
  explicit PROTOBUF_CONSTEXPR Joint_Gearbox(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Joint_Gearbox(const Joint_Gearbox& from);
  Joint_Gearbox(Joint_Gearbox&& from) noexcept
    : Joint_Gearbox() {
    *this = ::std::move(from);
  }

  inline Joint_Gearbox& operator=(const Joint_Gearbox& from) {
    CopyFrom(from);
    return *this;
  }
  inline Joint_Gearbox& operator=(Joint_Gearbox&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Joint_Gearbox& default_instance() {
    return *internal_default_instance();
  }
  static inline const Joint_Gearbox* internal_default_instance() {
    return reinterpret_cast<const Joint_Gearbox*>(
               &_Joint_Gearbox_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    0;

  friend void swap(Joint_Gearbox& a, Joint_Gearbox& b) {
    a.Swap(&b);
  }
  inline void Swap(Joint_Gearbox* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Joint_Gearbox* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Joint_Gearbox* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<Joint_Gearbox>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const Joint_Gearbox& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom(const Joint_Gearbox& from);
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message* to, const ::PROTOBUF_NAMESPACE_ID::Message& from);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  void SharedCtor();
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Joint_Gearbox* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.Joint.Gearbox";
  }
  protected:
  explicit Joint_Gearbox(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kGearboxReferenceBodyFieldNumber = 1,
    kGearboxRatioFieldNumber = 2,
  };
  // required string gearbox_reference_body = 1;
  bool has_gearbox_reference_body() const;
  private:
  bool _internal_has_gearbox_reference_body() const;
  public:
  void clear_gearbox_reference_body();
  const std::string& gearbox_reference_body() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_gearbox_reference_body(ArgT0&& arg0, ArgT... args);
  std::string* mutable_gearbox_reference_body();
  PROTOBUF_NODISCARD std::string* release_gearbox_reference_body();
  void set_allocated_gearbox_reference_body(std::string* gearbox_reference_body);
  private:
  const std::string& _internal_gearbox_reference_body() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_gearbox_reference_body(const std::string& value);
  std::string* _internal_mutable_gearbox_reference_body();
  public:

  // required double gearbox_ratio = 2;
  bool has_gearbox_ratio() const;
  private:
  bool _internal_has_gearbox_ratio() const;
  public:
  void clear_gearbox_ratio();
  double gearbox_ratio() const;
  void set_gearbox_ratio(double value);
  private:
  double _internal_gearbox_ratio() const;
  void _internal_set_gearbox_ratio(double value);
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.Joint.Gearbox)
 private:
  class _Internal;

  // helper for ByteSizeLong()
  size_t RequiredFieldsByteSizeFallback() const;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr gearbox_reference_body_;
  double gearbox_ratio_;
  friend struct ::TableStruct_joint_2eproto;
};
// -------------------------------------------------------------------

class GZ_MSGS_VISIBLE Joint_Screw final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.Joint.Screw) */ {
 public:
  inline Joint_Screw() : Joint_Screw(nullptr) {}
  ~Joint_Screw() override;
  explicit PROTOBUF_CONSTEXPR Joint_Screw(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Joint_Screw(const Joint_Screw& from);
  Joint_Screw(Joint_Screw&& from) noexcept
    : Joint_Screw() {
    *this = ::std::move(from);
  }

  inline Joint_Screw& operator=(const Joint_Screw& from) {
    CopyFrom(from);
    return *this;
  }
  inline Joint_Screw& operator=(Joint_Screw&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Joint_Screw& default_instance() {
    return *internal_default_instance();
  }
  static inline const Joint_Screw* internal_default_instance() {
    return reinterpret_cast<const Joint_Screw*>(
               &_Joint_Screw_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    1;

  friend void swap(Joint_Screw& a, Joint_Screw& b) {
    a.Swap(&b);
  }
  inline void Swap(Joint_Screw* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Joint_Screw* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Joint_Screw* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<Joint_Screw>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const Joint_Screw& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom(const Joint_Screw& from);
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message* to, const ::PROTOBUF_NAMESPACE_ID::Message& from);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  void SharedCtor();
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Joint_Screw* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.Joint.Screw";
  }
  protected:
  explicit Joint_Screw(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------

  enum : int {
    kThreadPitchFieldNumber = 1,
  };
  // required double thread_pitch = 1;
  bool has_thread_pitch() const;
  private:
  bool _internal_has_thread_pitch() const;
  public:
  void clear_thread_pitch();
  double thread_pitch() const;
  void set_thread_pitch(double value);
  private:
  double _internal_thread_pitch() const;
  void _internal_set_thread_pitch(double value);
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.Joint.Screw)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  double thread_pitch_;
  friend struct ::TableStruct_joint_2eproto;
};
// -------------------------------------------------------------------

class GZ_MSGS_VISIBLE Joint final :
    public ::PROTOBUF_NAMESPACE_ID::Message /* @@protoc_insertion_point(class_definition:gazebo.msgs.Joint) */ {
 public:
  inline Joint() : Joint(nullptr) {}
  ~Joint() override;
  explicit PROTOBUF_CONSTEXPR Joint(::PROTOBUF_NAMESPACE_ID::internal::ConstantInitialized);

  Joint(const Joint& from);
  Joint(Joint&& from) noexcept
    : Joint() {
    *this = ::std::move(from);
  }

  inline Joint& operator=(const Joint& from) {
    CopyFrom(from);
    return *this;
  }
  inline Joint& operator=(Joint&& from) noexcept {
    if (this == &from) return *this;
    if (GetOwningArena() == from.GetOwningArena()
  #ifdef PROTOBUF_FORCE_COPY_IN_MOVE
        && GetOwningArena() != nullptr
  #endif  // !PROTOBUF_FORCE_COPY_IN_MOVE
    ) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  inline const ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet& unknown_fields() const {
    return _internal_metadata_.unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>(::PROTOBUF_NAMESPACE_ID::UnknownFieldSet::default_instance);
  }
  inline ::PROTOBUF_NAMESPACE_ID::UnknownFieldSet* mutable_unknown_fields() {
    return _internal_metadata_.mutable_unknown_fields<::PROTOBUF_NAMESPACE_ID::UnknownFieldSet>();
  }

  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* descriptor() {
    return GetDescriptor();
  }
  static const ::PROTOBUF_NAMESPACE_ID::Descriptor* GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  static const ::PROTOBUF_NAMESPACE_ID::Reflection* GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  static const Joint& default_instance() {
    return *internal_default_instance();
  }
  static inline const Joint* internal_default_instance() {
    return reinterpret_cast<const Joint*>(
               &_Joint_default_instance_);
  }
  static constexpr int kIndexInFileMessages =
    2;

  friend void swap(Joint& a, Joint& b) {
    a.Swap(&b);
  }
  inline void Swap(Joint* other) {
    if (other == this) return;
  #ifdef PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() != nullptr &&
        GetOwningArena() == other->GetOwningArena()) {
   #else  // PROTOBUF_FORCE_COPY_IN_SWAP
    if (GetOwningArena() == other->GetOwningArena()) {
  #endif  // !PROTOBUF_FORCE_COPY_IN_SWAP
      InternalSwap(other);
    } else {
      ::PROTOBUF_NAMESPACE_ID::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Joint* other) {
    if (other == this) return;
    GOOGLE_DCHECK(GetOwningArena() == other->GetOwningArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  Joint* New(::PROTOBUF_NAMESPACE_ID::Arena* arena = nullptr) const final {
    return CreateMaybeMessage<Joint>(arena);
  }
  using ::PROTOBUF_NAMESPACE_ID::Message::CopyFrom;
  void CopyFrom(const Joint& from);
  using ::PROTOBUF_NAMESPACE_ID::Message::MergeFrom;
  void MergeFrom(const Joint& from);
  private:
  static void MergeImpl(::PROTOBUF_NAMESPACE_ID::Message* to, const ::PROTOBUF_NAMESPACE_ID::Message& from);
  public:
  PROTOBUF_ATTRIBUTE_REINITIALIZES void Clear() final;
  bool IsInitialized() const final;

  size_t ByteSizeLong() const final;
  const char* _InternalParse(const char* ptr, ::PROTOBUF_NAMESPACE_ID::internal::ParseContext* ctx) final;
  uint8_t* _InternalSerialize(
      uint8_t* target, ::PROTOBUF_NAMESPACE_ID::io::EpsCopyOutputStream* stream) const final;
  int GetCachedSize() const final { return _cached_size_.Get(); }

  private:
  void SharedCtor();
  void SharedDtor();
  void SetCachedSize(int size) const final;
  void InternalSwap(Joint* other);

  private:
  friend class ::PROTOBUF_NAMESPACE_ID::internal::AnyMetadata;
  static ::PROTOBUF_NAMESPACE_ID::StringPiece FullMessageName() {
    return "gazebo.msgs.Joint";
  }
  protected:
  explicit Joint(::PROTOBUF_NAMESPACE_ID::Arena* arena,
                       bool is_message_owned = false);
  public:

  static const ClassData _class_data_;
  const ::PROTOBUF_NAMESPACE_ID::Message::ClassData*GetClassData() const final;

  ::PROTOBUF_NAMESPACE_ID::Metadata GetMetadata() const final;

  // nested types ----------------------------------------------------

  typedef Joint_Gearbox Gearbox;
  typedef Joint_Screw Screw;

  typedef Joint_Type Type;
  static constexpr Type REVOLUTE =
    Joint_Type_REVOLUTE;
  static constexpr Type REVOLUTE2 =
    Joint_Type_REVOLUTE2;
  static constexpr Type PRISMATIC =
    Joint_Type_PRISMATIC;
  static constexpr Type UNIVERSAL =
    Joint_Type_UNIVERSAL;
  static constexpr Type BALL =
    Joint_Type_BALL;
  static constexpr Type SCREW =
    Joint_Type_SCREW;
  static constexpr Type GEARBOX =
    Joint_Type_GEARBOX;
  static constexpr Type FIXED =
    Joint_Type_FIXED;
  static inline bool Type_IsValid(int value) {
    return Joint_Type_IsValid(value);
  }
  static constexpr Type Type_MIN =
    Joint_Type_Type_MIN;
  static constexpr Type Type_MAX =
    Joint_Type_Type_MAX;
  static constexpr int Type_ARRAYSIZE =
    Joint_Type_Type_ARRAYSIZE;
  static inline const ::PROTOBUF_NAMESPACE_ID::EnumDescriptor*
  Type_descriptor() {
    return Joint_Type_descriptor();
  }
  template<typename T>
  static inline const std::string& Type_Name(T enum_t_value) {
    static_assert(::std::is_same<T, Type>::value ||
      ::std::is_integral<T>::value,
      "Incorrect type passed to function Type_Name.");
    return Joint_Type_Name(enum_t_value);
  }
  static inline bool Type_Parse(::PROTOBUF_NAMESPACE_ID::ConstStringParam name,
      Type* value) {
    return Joint_Type_Parse(name, value);
  }

  // accessors -------------------------------------------------------

  enum : int {
    kAngleFieldNumber = 3,
    kSensorFieldNumber = 22,
    kNameFieldNumber = 1,
    kParentFieldNumber = 5,
    kChildFieldNumber = 7,
    kPoseFieldNumber = 9,
    kAxis1FieldNumber = 10,
    kAxis2FieldNumber = 11,
    kGearboxFieldNumber = 20,
    kScrewFieldNumber = 21,
    kIdFieldNumber = 2,
    kParentIdFieldNumber = 6,
    kCfmFieldNumber = 12,
    kBounceFieldNumber = 13,
    kVelocityFieldNumber = 14,
    kFudgeFactorFieldNumber = 15,
    kLimitCfmFieldNumber = 16,
    kLimitErpFieldNumber = 17,
    kSuspensionCfmFieldNumber = 18,
    kSuspensionErpFieldNumber = 19,
    kChildIdFieldNumber = 8,
    kTypeFieldNumber = 4,
  };
  // repeated double angle = 3;
  int angle_size() const;
  private:
  int _internal_angle_size() const;
  public:
  void clear_angle();
  private:
  double _internal_angle(int index) const;
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >&
      _internal_angle() const;
  void _internal_add_angle(double value);
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >*
      _internal_mutable_angle();
  public:
  double angle(int index) const;
  void set_angle(int index, double value);
  void add_angle(double value);
  const ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >&
      angle() const;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >*
      mutable_angle();

  // repeated .gazebo.msgs.Sensor sensor = 22;
  int sensor_size() const;
  private:
  int _internal_sensor_size() const;
  public:
  void clear_sensor();
  ::gazebo::msgs::Sensor* mutable_sensor(int index);
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Sensor >*
      mutable_sensor();
  private:
  const ::gazebo::msgs::Sensor& _internal_sensor(int index) const;
  ::gazebo::msgs::Sensor* _internal_add_sensor();
  public:
  const ::gazebo::msgs::Sensor& sensor(int index) const;
  ::gazebo::msgs::Sensor* add_sensor();
  const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Sensor >&
      sensor() const;

  // required string name = 1;
  bool has_name() const;
  private:
  bool _internal_has_name() const;
  public:
  void clear_name();
  const std::string& name() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_name(ArgT0&& arg0, ArgT... args);
  std::string* mutable_name();
  PROTOBUF_NODISCARD std::string* release_name();
  void set_allocated_name(std::string* name);
  private:
  const std::string& _internal_name() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_name(const std::string& value);
  std::string* _internal_mutable_name();
  public:

  // optional string parent = 5;
  bool has_parent() const;
  private:
  bool _internal_has_parent() const;
  public:
  void clear_parent();
  const std::string& parent() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_parent(ArgT0&& arg0, ArgT... args);
  std::string* mutable_parent();
  PROTOBUF_NODISCARD std::string* release_parent();
  void set_allocated_parent(std::string* parent);
  private:
  const std::string& _internal_parent() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_parent(const std::string& value);
  std::string* _internal_mutable_parent();
  public:

  // optional string child = 7;
  bool has_child() const;
  private:
  bool _internal_has_child() const;
  public:
  void clear_child();
  const std::string& child() const;
  template <typename ArgT0 = const std::string&, typename... ArgT>
  void set_child(ArgT0&& arg0, ArgT... args);
  std::string* mutable_child();
  PROTOBUF_NODISCARD std::string* release_child();
  void set_allocated_child(std::string* child);
  private:
  const std::string& _internal_child() const;
  inline PROTOBUF_ALWAYS_INLINE void _internal_set_child(const std::string& value);
  std::string* _internal_mutable_child();
  public:

  // optional .gazebo.msgs.Pose pose = 9;
  bool has_pose() const;
  private:
  bool _internal_has_pose() const;
  public:
  void clear_pose();
  const ::gazebo::msgs::Pose& pose() const;
  PROTOBUF_NODISCARD ::gazebo::msgs::Pose* release_pose();
  ::gazebo::msgs::Pose* mutable_pose();
  void set_allocated_pose(::gazebo::msgs::Pose* pose);
  private:
  const ::gazebo::msgs::Pose& _internal_pose() const;
  ::gazebo::msgs::Pose* _internal_mutable_pose();
  public:
  void unsafe_arena_set_allocated_pose(
      ::gazebo::msgs::Pose* pose);
  ::gazebo::msgs::Pose* unsafe_arena_release_pose();

  // optional .gazebo.msgs.Axis axis1 = 10;
  bool has_axis1() const;
  private:
  bool _internal_has_axis1() const;
  public:
  void clear_axis1();
  const ::gazebo::msgs::Axis& axis1() const;
  PROTOBUF_NODISCARD ::gazebo::msgs::Axis* release_axis1();
  ::gazebo::msgs::Axis* mutable_axis1();
  void set_allocated_axis1(::gazebo::msgs::Axis* axis1);
  private:
  const ::gazebo::msgs::Axis& _internal_axis1() const;
  ::gazebo::msgs::Axis* _internal_mutable_axis1();
  public:
  void unsafe_arena_set_allocated_axis1(
      ::gazebo::msgs::Axis* axis1);
  ::gazebo::msgs::Axis* unsafe_arena_release_axis1();

  // optional .gazebo.msgs.Axis axis2 = 11;
  bool has_axis2() const;
  private:
  bool _internal_has_axis2() const;
  public:
  void clear_axis2();
  const ::gazebo::msgs::Axis& axis2() const;
  PROTOBUF_NODISCARD ::gazebo::msgs::Axis* release_axis2();
  ::gazebo::msgs::Axis* mutable_axis2();
  void set_allocated_axis2(::gazebo::msgs::Axis* axis2);
  private:
  const ::gazebo::msgs::Axis& _internal_axis2() const;
  ::gazebo::msgs::Axis* _internal_mutable_axis2();
  public:
  void unsafe_arena_set_allocated_axis2(
      ::gazebo::msgs::Axis* axis2);
  ::gazebo::msgs::Axis* unsafe_arena_release_axis2();

  // optional .gazebo.msgs.Joint.Gearbox gearbox = 20;
  bool has_gearbox() const;
  private:
  bool _internal_has_gearbox() const;
  public:
  void clear_gearbox();
  const ::gazebo::msgs::Joint_Gearbox& gearbox() const;
  PROTOBUF_NODISCARD ::gazebo::msgs::Joint_Gearbox* release_gearbox();
  ::gazebo::msgs::Joint_Gearbox* mutable_gearbox();
  void set_allocated_gearbox(::gazebo::msgs::Joint_Gearbox* gearbox);
  private:
  const ::gazebo::msgs::Joint_Gearbox& _internal_gearbox() const;
  ::gazebo::msgs::Joint_Gearbox* _internal_mutable_gearbox();
  public:
  void unsafe_arena_set_allocated_gearbox(
      ::gazebo::msgs::Joint_Gearbox* gearbox);
  ::gazebo::msgs::Joint_Gearbox* unsafe_arena_release_gearbox();

  // optional .gazebo.msgs.Joint.Screw screw = 21;
  bool has_screw() const;
  private:
  bool _internal_has_screw() const;
  public:
  void clear_screw();
  const ::gazebo::msgs::Joint_Screw& screw() const;
  PROTOBUF_NODISCARD ::gazebo::msgs::Joint_Screw* release_screw();
  ::gazebo::msgs::Joint_Screw* mutable_screw();
  void set_allocated_screw(::gazebo::msgs::Joint_Screw* screw);
  private:
  const ::gazebo::msgs::Joint_Screw& _internal_screw() const;
  ::gazebo::msgs::Joint_Screw* _internal_mutable_screw();
  public:
  void unsafe_arena_set_allocated_screw(
      ::gazebo::msgs::Joint_Screw* screw);
  ::gazebo::msgs::Joint_Screw* unsafe_arena_release_screw();

  // optional uint32 id = 2;
  bool has_id() const;
  private:
  bool _internal_has_id() const;
  public:
  void clear_id();
  uint32_t id() const;
  void set_id(uint32_t value);
  private:
  uint32_t _internal_id() const;
  void _internal_set_id(uint32_t value);
  public:

  // optional uint32 parent_id = 6;
  bool has_parent_id() const;
  private:
  bool _internal_has_parent_id() const;
  public:
  void clear_parent_id();
  uint32_t parent_id() const;
  void set_parent_id(uint32_t value);
  private:
  uint32_t _internal_parent_id() const;
  void _internal_set_parent_id(uint32_t value);
  public:

  // optional double cfm = 12;
  bool has_cfm() const;
  private:
  bool _internal_has_cfm() const;
  public:
  void clear_cfm();
  double cfm() const;
  void set_cfm(double value);
  private:
  double _internal_cfm() const;
  void _internal_set_cfm(double value);
  public:

  // optional double bounce = 13;
  bool has_bounce() const;
  private:
  bool _internal_has_bounce() const;
  public:
  void clear_bounce();
  double bounce() const;
  void set_bounce(double value);
  private:
  double _internal_bounce() const;
  void _internal_set_bounce(double value);
  public:

  // optional double velocity = 14;
  bool has_velocity() const;
  private:
  bool _internal_has_velocity() const;
  public:
  void clear_velocity();
  double velocity() const;
  void set_velocity(double value);
  private:
  double _internal_velocity() const;
  void _internal_set_velocity(double value);
  public:

  // optional double fudge_factor = 15;
  bool has_fudge_factor() const;
  private:
  bool _internal_has_fudge_factor() const;
  public:
  void clear_fudge_factor();
  double fudge_factor() const;
  void set_fudge_factor(double value);
  private:
  double _internal_fudge_factor() const;
  void _internal_set_fudge_factor(double value);
  public:

  // optional double limit_cfm = 16;
  bool has_limit_cfm() const;
  private:
  bool _internal_has_limit_cfm() const;
  public:
  void clear_limit_cfm();
  double limit_cfm() const;
  void set_limit_cfm(double value);
  private:
  double _internal_limit_cfm() const;
  void _internal_set_limit_cfm(double value);
  public:

  // optional double limit_erp = 17;
  bool has_limit_erp() const;
  private:
  bool _internal_has_limit_erp() const;
  public:
  void clear_limit_erp();
  double limit_erp() const;
  void set_limit_erp(double value);
  private:
  double _internal_limit_erp() const;
  void _internal_set_limit_erp(double value);
  public:

  // optional double suspension_cfm = 18;
  bool has_suspension_cfm() const;
  private:
  bool _internal_has_suspension_cfm() const;
  public:
  void clear_suspension_cfm();
  double suspension_cfm() const;
  void set_suspension_cfm(double value);
  private:
  double _internal_suspension_cfm() const;
  void _internal_set_suspension_cfm(double value);
  public:

  // optional double suspension_erp = 19;
  bool has_suspension_erp() const;
  private:
  bool _internal_has_suspension_erp() const;
  public:
  void clear_suspension_erp();
  double suspension_erp() const;
  void set_suspension_erp(double value);
  private:
  double _internal_suspension_erp() const;
  void _internal_set_suspension_erp(double value);
  public:

  // optional uint32 child_id = 8;
  bool has_child_id() const;
  private:
  bool _internal_has_child_id() const;
  public:
  void clear_child_id();
  uint32_t child_id() const;
  void set_child_id(uint32_t value);
  private:
  uint32_t _internal_child_id() const;
  void _internal_set_child_id(uint32_t value);
  public:

  // optional .gazebo.msgs.Joint.Type type = 4;
  bool has_type() const;
  private:
  bool _internal_has_type() const;
  public:
  void clear_type();
  ::gazebo::msgs::Joint_Type type() const;
  void set_type(::gazebo::msgs::Joint_Type value);
  private:
  ::gazebo::msgs::Joint_Type _internal_type() const;
  void _internal_set_type(::gazebo::msgs::Joint_Type value);
  public:

  // @@protoc_insertion_point(class_scope:gazebo.msgs.Joint)
 private:
  class _Internal;

  template <typename T> friend class ::PROTOBUF_NAMESPACE_ID::Arena::InternalHelper;
  typedef void InternalArenaConstructable_;
  typedef void DestructorSkippable_;
  ::PROTOBUF_NAMESPACE_ID::internal::HasBits<1> _has_bits_;
  mutable ::PROTOBUF_NAMESPACE_ID::internal::CachedSize _cached_size_;
  ::PROTOBUF_NAMESPACE_ID::RepeatedField< double > angle_;
  ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Sensor > sensor_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr name_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr parent_;
  ::PROTOBUF_NAMESPACE_ID::internal::ArenaStringPtr child_;
  ::gazebo::msgs::Pose* pose_;
  ::gazebo::msgs::Axis* axis1_;
  ::gazebo::msgs::Axis* axis2_;
  ::gazebo::msgs::Joint_Gearbox* gearbox_;
  ::gazebo::msgs::Joint_Screw* screw_;
  uint32_t id_;
  uint32_t parent_id_;
  double cfm_;
  double bounce_;
  double velocity_;
  double fudge_factor_;
  double limit_cfm_;
  double limit_erp_;
  double suspension_cfm_;
  double suspension_erp_;
  uint32_t child_id_;
  int type_;
  friend struct ::TableStruct_joint_2eproto;
};
// ===================================================================


// ===================================================================

#ifdef __GNUC__
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// Joint_Gearbox

// required string gearbox_reference_body = 1;
inline bool Joint_Gearbox::_internal_has_gearbox_reference_body() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool Joint_Gearbox::has_gearbox_reference_body() const {
  return _internal_has_gearbox_reference_body();
}
inline void Joint_Gearbox::clear_gearbox_reference_body() {
  gearbox_reference_body_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000001u;
}
inline const std::string& Joint_Gearbox::gearbox_reference_body() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.Gearbox.gearbox_reference_body)
  return _internal_gearbox_reference_body();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void Joint_Gearbox::set_gearbox_reference_body(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000001u;
 gearbox_reference_body_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.Gearbox.gearbox_reference_body)
}
inline std::string* Joint_Gearbox::mutable_gearbox_reference_body() {
  std::string* _s = _internal_mutable_gearbox_reference_body();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.Gearbox.gearbox_reference_body)
  return _s;
}
inline const std::string& Joint_Gearbox::_internal_gearbox_reference_body() const {
  return gearbox_reference_body_.Get();
}
inline void Joint_Gearbox::_internal_set_gearbox_reference_body(const std::string& value) {
  _has_bits_[0] |= 0x00000001u;
  gearbox_reference_body_.Set(value, GetArenaForAllocation());
}
inline std::string* Joint_Gearbox::_internal_mutable_gearbox_reference_body() {
  _has_bits_[0] |= 0x00000001u;
  return gearbox_reference_body_.Mutable(GetArenaForAllocation());
}
inline std::string* Joint_Gearbox::release_gearbox_reference_body() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Joint.Gearbox.gearbox_reference_body)
  if (!_internal_has_gearbox_reference_body()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000001u;
  auto* p = gearbox_reference_body_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (gearbox_reference_body_.IsDefault()) {
    gearbox_reference_body_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void Joint_Gearbox::set_allocated_gearbox_reference_body(std::string* gearbox_reference_body) {
  if (gearbox_reference_body != nullptr) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  gearbox_reference_body_.SetAllocated(gearbox_reference_body, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (gearbox_reference_body_.IsDefault()) {
    gearbox_reference_body_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Joint.Gearbox.gearbox_reference_body)
}

// required double gearbox_ratio = 2;
inline bool Joint_Gearbox::_internal_has_gearbox_ratio() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool Joint_Gearbox::has_gearbox_ratio() const {
  return _internal_has_gearbox_ratio();
}
inline void Joint_Gearbox::clear_gearbox_ratio() {
  gearbox_ratio_ = 0;
  _has_bits_[0] &= ~0x00000002u;
}
inline double Joint_Gearbox::_internal_gearbox_ratio() const {
  return gearbox_ratio_;
}
inline double Joint_Gearbox::gearbox_ratio() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.Gearbox.gearbox_ratio)
  return _internal_gearbox_ratio();
}
inline void Joint_Gearbox::_internal_set_gearbox_ratio(double value) {
  _has_bits_[0] |= 0x00000002u;
  gearbox_ratio_ = value;
}
inline void Joint_Gearbox::set_gearbox_ratio(double value) {
  _internal_set_gearbox_ratio(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.Gearbox.gearbox_ratio)
}

// -------------------------------------------------------------------

// Joint_Screw

// required double thread_pitch = 1;
inline bool Joint_Screw::_internal_has_thread_pitch() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool Joint_Screw::has_thread_pitch() const {
  return _internal_has_thread_pitch();
}
inline void Joint_Screw::clear_thread_pitch() {
  thread_pitch_ = 0;
  _has_bits_[0] &= ~0x00000001u;
}
inline double Joint_Screw::_internal_thread_pitch() const {
  return thread_pitch_;
}
inline double Joint_Screw::thread_pitch() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.Screw.thread_pitch)
  return _internal_thread_pitch();
}
inline void Joint_Screw::_internal_set_thread_pitch(double value) {
  _has_bits_[0] |= 0x00000001u;
  thread_pitch_ = value;
}
inline void Joint_Screw::set_thread_pitch(double value) {
  _internal_set_thread_pitch(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.Screw.thread_pitch)
}

// -------------------------------------------------------------------

// Joint

// required string name = 1;
inline bool Joint::_internal_has_name() const {
  bool value = (_has_bits_[0] & 0x00000001u) != 0;
  return value;
}
inline bool Joint::has_name() const {
  return _internal_has_name();
}
inline void Joint::clear_name() {
  name_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000001u;
}
inline const std::string& Joint::name() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.name)
  return _internal_name();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void Joint::set_name(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000001u;
 name_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.name)
}
inline std::string* Joint::mutable_name() {
  std::string* _s = _internal_mutable_name();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.name)
  return _s;
}
inline const std::string& Joint::_internal_name() const {
  return name_.Get();
}
inline void Joint::_internal_set_name(const std::string& value) {
  _has_bits_[0] |= 0x00000001u;
  name_.Set(value, GetArenaForAllocation());
}
inline std::string* Joint::_internal_mutable_name() {
  _has_bits_[0] |= 0x00000001u;
  return name_.Mutable(GetArenaForAllocation());
}
inline std::string* Joint::release_name() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Joint.name)
  if (!_internal_has_name()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000001u;
  auto* p = name_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (name_.IsDefault()) {
    name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void Joint::set_allocated_name(std::string* name) {
  if (name != nullptr) {
    _has_bits_[0] |= 0x00000001u;
  } else {
    _has_bits_[0] &= ~0x00000001u;
  }
  name_.SetAllocated(name, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (name_.IsDefault()) {
    name_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Joint.name)
}

// optional uint32 id = 2;
inline bool Joint::_internal_has_id() const {
  bool value = (_has_bits_[0] & 0x00000100u) != 0;
  return value;
}
inline bool Joint::has_id() const {
  return _internal_has_id();
}
inline void Joint::clear_id() {
  id_ = 0u;
  _has_bits_[0] &= ~0x00000100u;
}
inline uint32_t Joint::_internal_id() const {
  return id_;
}
inline uint32_t Joint::id() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.id)
  return _internal_id();
}
inline void Joint::_internal_set_id(uint32_t value) {
  _has_bits_[0] |= 0x00000100u;
  id_ = value;
}
inline void Joint::set_id(uint32_t value) {
  _internal_set_id(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.id)
}

// repeated double angle = 3;
inline int Joint::_internal_angle_size() const {
  return angle_.size();
}
inline int Joint::angle_size() const {
  return _internal_angle_size();
}
inline void Joint::clear_angle() {
  angle_.Clear();
}
inline double Joint::_internal_angle(int index) const {
  return angle_.Get(index);
}
inline double Joint::angle(int index) const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.angle)
  return _internal_angle(index);
}
inline void Joint::set_angle(int index, double value) {
  angle_.Set(index, value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.angle)
}
inline void Joint::_internal_add_angle(double value) {
  angle_.Add(value);
}
inline void Joint::add_angle(double value) {
  _internal_add_angle(value);
  // @@protoc_insertion_point(field_add:gazebo.msgs.Joint.angle)
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >&
Joint::_internal_angle() const {
  return angle_;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >&
Joint::angle() const {
  // @@protoc_insertion_point(field_list:gazebo.msgs.Joint.angle)
  return _internal_angle();
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >*
Joint::_internal_mutable_angle() {
  return &angle_;
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedField< double >*
Joint::mutable_angle() {
  // @@protoc_insertion_point(field_mutable_list:gazebo.msgs.Joint.angle)
  return _internal_mutable_angle();
}

// optional .gazebo.msgs.Joint.Type type = 4;
inline bool Joint::_internal_has_type() const {
  bool value = (_has_bits_[0] & 0x00080000u) != 0;
  return value;
}
inline bool Joint::has_type() const {
  return _internal_has_type();
}
inline void Joint::clear_type() {
  type_ = 1;
  _has_bits_[0] &= ~0x00080000u;
}
inline ::gazebo::msgs::Joint_Type Joint::_internal_type() const {
  return static_cast< ::gazebo::msgs::Joint_Type >(type_);
}
inline ::gazebo::msgs::Joint_Type Joint::type() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.type)
  return _internal_type();
}
inline void Joint::_internal_set_type(::gazebo::msgs::Joint_Type value) {
  assert(::gazebo::msgs::Joint_Type_IsValid(value));
  _has_bits_[0] |= 0x00080000u;
  type_ = value;
}
inline void Joint::set_type(::gazebo::msgs::Joint_Type value) {
  _internal_set_type(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.type)
}

// optional string parent = 5;
inline bool Joint::_internal_has_parent() const {
  bool value = (_has_bits_[0] & 0x00000002u) != 0;
  return value;
}
inline bool Joint::has_parent() const {
  return _internal_has_parent();
}
inline void Joint::clear_parent() {
  parent_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000002u;
}
inline const std::string& Joint::parent() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.parent)
  return _internal_parent();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void Joint::set_parent(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000002u;
 parent_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.parent)
}
inline std::string* Joint::mutable_parent() {
  std::string* _s = _internal_mutable_parent();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.parent)
  return _s;
}
inline const std::string& Joint::_internal_parent() const {
  return parent_.Get();
}
inline void Joint::_internal_set_parent(const std::string& value) {
  _has_bits_[0] |= 0x00000002u;
  parent_.Set(value, GetArenaForAllocation());
}
inline std::string* Joint::_internal_mutable_parent() {
  _has_bits_[0] |= 0x00000002u;
  return parent_.Mutable(GetArenaForAllocation());
}
inline std::string* Joint::release_parent() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Joint.parent)
  if (!_internal_has_parent()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000002u;
  auto* p = parent_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (parent_.IsDefault()) {
    parent_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void Joint::set_allocated_parent(std::string* parent) {
  if (parent != nullptr) {
    _has_bits_[0] |= 0x00000002u;
  } else {
    _has_bits_[0] &= ~0x00000002u;
  }
  parent_.SetAllocated(parent, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (parent_.IsDefault()) {
    parent_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Joint.parent)
}

// optional uint32 parent_id = 6;
inline bool Joint::_internal_has_parent_id() const {
  bool value = (_has_bits_[0] & 0x00000200u) != 0;
  return value;
}
inline bool Joint::has_parent_id() const {
  return _internal_has_parent_id();
}
inline void Joint::clear_parent_id() {
  parent_id_ = 0u;
  _has_bits_[0] &= ~0x00000200u;
}
inline uint32_t Joint::_internal_parent_id() const {
  return parent_id_;
}
inline uint32_t Joint::parent_id() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.parent_id)
  return _internal_parent_id();
}
inline void Joint::_internal_set_parent_id(uint32_t value) {
  _has_bits_[0] |= 0x00000200u;
  parent_id_ = value;
}
inline void Joint::set_parent_id(uint32_t value) {
  _internal_set_parent_id(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.parent_id)
}

// optional string child = 7;
inline bool Joint::_internal_has_child() const {
  bool value = (_has_bits_[0] & 0x00000004u) != 0;
  return value;
}
inline bool Joint::has_child() const {
  return _internal_has_child();
}
inline void Joint::clear_child() {
  child_.ClearToEmpty();
  _has_bits_[0] &= ~0x00000004u;
}
inline const std::string& Joint::child() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.child)
  return _internal_child();
}
template <typename ArgT0, typename... ArgT>
inline PROTOBUF_ALWAYS_INLINE
void Joint::set_child(ArgT0&& arg0, ArgT... args) {
 _has_bits_[0] |= 0x00000004u;
 child_.Set(static_cast<ArgT0 &&>(arg0), args..., GetArenaForAllocation());
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.child)
}
inline std::string* Joint::mutable_child() {
  std::string* _s = _internal_mutable_child();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.child)
  return _s;
}
inline const std::string& Joint::_internal_child() const {
  return child_.Get();
}
inline void Joint::_internal_set_child(const std::string& value) {
  _has_bits_[0] |= 0x00000004u;
  child_.Set(value, GetArenaForAllocation());
}
inline std::string* Joint::_internal_mutable_child() {
  _has_bits_[0] |= 0x00000004u;
  return child_.Mutable(GetArenaForAllocation());
}
inline std::string* Joint::release_child() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Joint.child)
  if (!_internal_has_child()) {
    return nullptr;
  }
  _has_bits_[0] &= ~0x00000004u;
  auto* p = child_.Release();
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (child_.IsDefault()) {
    child_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  return p;
}
inline void Joint::set_allocated_child(std::string* child) {
  if (child != nullptr) {
    _has_bits_[0] |= 0x00000004u;
  } else {
    _has_bits_[0] &= ~0x00000004u;
  }
  child_.SetAllocated(child, GetArenaForAllocation());
#ifdef PROTOBUF_FORCE_COPY_DEFAULT_STRING
  if (child_.IsDefault()) {
    child_.Set("", GetArenaForAllocation());
  }
#endif // PROTOBUF_FORCE_COPY_DEFAULT_STRING
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Joint.child)
}

// optional uint32 child_id = 8;
inline bool Joint::_internal_has_child_id() const {
  bool value = (_has_bits_[0] & 0x00040000u) != 0;
  return value;
}
inline bool Joint::has_child_id() const {
  return _internal_has_child_id();
}
inline void Joint::clear_child_id() {
  child_id_ = 0u;
  _has_bits_[0] &= ~0x00040000u;
}
inline uint32_t Joint::_internal_child_id() const {
  return child_id_;
}
inline uint32_t Joint::child_id() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.child_id)
  return _internal_child_id();
}
inline void Joint::_internal_set_child_id(uint32_t value) {
  _has_bits_[0] |= 0x00040000u;
  child_id_ = value;
}
inline void Joint::set_child_id(uint32_t value) {
  _internal_set_child_id(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.child_id)
}

// optional .gazebo.msgs.Pose pose = 9;
inline bool Joint::_internal_has_pose() const {
  bool value = (_has_bits_[0] & 0x00000008u) != 0;
  PROTOBUF_ASSUME(!value || pose_ != nullptr);
  return value;
}
inline bool Joint::has_pose() const {
  return _internal_has_pose();
}
inline const ::gazebo::msgs::Pose& Joint::_internal_pose() const {
  const ::gazebo::msgs::Pose* p = pose_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Pose&>(
      ::gazebo::msgs::_Pose_default_instance_);
}
inline const ::gazebo::msgs::Pose& Joint::pose() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.pose)
  return _internal_pose();
}
inline void Joint::unsafe_arena_set_allocated_pose(
    ::gazebo::msgs::Pose* pose) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose_);
  }
  pose_ = pose;
  if (pose) {
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Joint.pose)
}
inline ::gazebo::msgs::Pose* Joint::release_pose() {
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Pose* temp = pose_;
  pose_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::gazebo::msgs::Pose* Joint::unsafe_arena_release_pose() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Joint.pose)
  _has_bits_[0] &= ~0x00000008u;
  ::gazebo::msgs::Pose* temp = pose_;
  pose_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Pose* Joint::_internal_mutable_pose() {
  _has_bits_[0] |= 0x00000008u;
  if (pose_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Pose>(GetArenaForAllocation());
    pose_ = p;
  }
  return pose_;
}
inline ::gazebo::msgs::Pose* Joint::mutable_pose() {
  ::gazebo::msgs::Pose* _msg = _internal_mutable_pose();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.pose)
  return _msg;
}
inline void Joint::set_allocated_pose(::gazebo::msgs::Pose* pose) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose_);
  }
  if (pose) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(pose));
    if (message_arena != submessage_arena) {
      pose = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, pose, submessage_arena);
    }
    _has_bits_[0] |= 0x00000008u;
  } else {
    _has_bits_[0] &= ~0x00000008u;
  }
  pose_ = pose;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Joint.pose)
}

// optional .gazebo.msgs.Axis axis1 = 10;
inline bool Joint::_internal_has_axis1() const {
  bool value = (_has_bits_[0] & 0x00000010u) != 0;
  PROTOBUF_ASSUME(!value || axis1_ != nullptr);
  return value;
}
inline bool Joint::has_axis1() const {
  return _internal_has_axis1();
}
inline const ::gazebo::msgs::Axis& Joint::_internal_axis1() const {
  const ::gazebo::msgs::Axis* p = axis1_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Axis&>(
      ::gazebo::msgs::_Axis_default_instance_);
}
inline const ::gazebo::msgs::Axis& Joint::axis1() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.axis1)
  return _internal_axis1();
}
inline void Joint::unsafe_arena_set_allocated_axis1(
    ::gazebo::msgs::Axis* axis1) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(axis1_);
  }
  axis1_ = axis1;
  if (axis1) {
    _has_bits_[0] |= 0x00000010u;
  } else {
    _has_bits_[0] &= ~0x00000010u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Joint.axis1)
}
inline ::gazebo::msgs::Axis* Joint::release_axis1() {
  _has_bits_[0] &= ~0x00000010u;
  ::gazebo::msgs::Axis* temp = axis1_;
  axis1_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::gazebo::msgs::Axis* Joint::unsafe_arena_release_axis1() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Joint.axis1)
  _has_bits_[0] &= ~0x00000010u;
  ::gazebo::msgs::Axis* temp = axis1_;
  axis1_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Axis* Joint::_internal_mutable_axis1() {
  _has_bits_[0] |= 0x00000010u;
  if (axis1_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Axis>(GetArenaForAllocation());
    axis1_ = p;
  }
  return axis1_;
}
inline ::gazebo::msgs::Axis* Joint::mutable_axis1() {
  ::gazebo::msgs::Axis* _msg = _internal_mutable_axis1();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.axis1)
  return _msg;
}
inline void Joint::set_allocated_axis1(::gazebo::msgs::Axis* axis1) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(axis1_);
  }
  if (axis1) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(axis1));
    if (message_arena != submessage_arena) {
      axis1 = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, axis1, submessage_arena);
    }
    _has_bits_[0] |= 0x00000010u;
  } else {
    _has_bits_[0] &= ~0x00000010u;
  }
  axis1_ = axis1;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Joint.axis1)
}

// optional .gazebo.msgs.Axis axis2 = 11;
inline bool Joint::_internal_has_axis2() const {
  bool value = (_has_bits_[0] & 0x00000020u) != 0;
  PROTOBUF_ASSUME(!value || axis2_ != nullptr);
  return value;
}
inline bool Joint::has_axis2() const {
  return _internal_has_axis2();
}
inline const ::gazebo::msgs::Axis& Joint::_internal_axis2() const {
  const ::gazebo::msgs::Axis* p = axis2_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Axis&>(
      ::gazebo::msgs::_Axis_default_instance_);
}
inline const ::gazebo::msgs::Axis& Joint::axis2() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.axis2)
  return _internal_axis2();
}
inline void Joint::unsafe_arena_set_allocated_axis2(
    ::gazebo::msgs::Axis* axis2) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(axis2_);
  }
  axis2_ = axis2;
  if (axis2) {
    _has_bits_[0] |= 0x00000020u;
  } else {
    _has_bits_[0] &= ~0x00000020u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Joint.axis2)
}
inline ::gazebo::msgs::Axis* Joint::release_axis2() {
  _has_bits_[0] &= ~0x00000020u;
  ::gazebo::msgs::Axis* temp = axis2_;
  axis2_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::gazebo::msgs::Axis* Joint::unsafe_arena_release_axis2() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Joint.axis2)
  _has_bits_[0] &= ~0x00000020u;
  ::gazebo::msgs::Axis* temp = axis2_;
  axis2_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Axis* Joint::_internal_mutable_axis2() {
  _has_bits_[0] |= 0x00000020u;
  if (axis2_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Axis>(GetArenaForAllocation());
    axis2_ = p;
  }
  return axis2_;
}
inline ::gazebo::msgs::Axis* Joint::mutable_axis2() {
  ::gazebo::msgs::Axis* _msg = _internal_mutable_axis2();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.axis2)
  return _msg;
}
inline void Joint::set_allocated_axis2(::gazebo::msgs::Axis* axis2) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete reinterpret_cast< ::PROTOBUF_NAMESPACE_ID::MessageLite*>(axis2_);
  }
  if (axis2) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(
                reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(axis2));
    if (message_arena != submessage_arena) {
      axis2 = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, axis2, submessage_arena);
    }
    _has_bits_[0] |= 0x00000020u;
  } else {
    _has_bits_[0] &= ~0x00000020u;
  }
  axis2_ = axis2;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Joint.axis2)
}

// optional double cfm = 12;
inline bool Joint::_internal_has_cfm() const {
  bool value = (_has_bits_[0] & 0x00000400u) != 0;
  return value;
}
inline bool Joint::has_cfm() const {
  return _internal_has_cfm();
}
inline void Joint::clear_cfm() {
  cfm_ = 0;
  _has_bits_[0] &= ~0x00000400u;
}
inline double Joint::_internal_cfm() const {
  return cfm_;
}
inline double Joint::cfm() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.cfm)
  return _internal_cfm();
}
inline void Joint::_internal_set_cfm(double value) {
  _has_bits_[0] |= 0x00000400u;
  cfm_ = value;
}
inline void Joint::set_cfm(double value) {
  _internal_set_cfm(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.cfm)
}

// optional double bounce = 13;
inline bool Joint::_internal_has_bounce() const {
  bool value = (_has_bits_[0] & 0x00000800u) != 0;
  return value;
}
inline bool Joint::has_bounce() const {
  return _internal_has_bounce();
}
inline void Joint::clear_bounce() {
  bounce_ = 0;
  _has_bits_[0] &= ~0x00000800u;
}
inline double Joint::_internal_bounce() const {
  return bounce_;
}
inline double Joint::bounce() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.bounce)
  return _internal_bounce();
}
inline void Joint::_internal_set_bounce(double value) {
  _has_bits_[0] |= 0x00000800u;
  bounce_ = value;
}
inline void Joint::set_bounce(double value) {
  _internal_set_bounce(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.bounce)
}

// optional double velocity = 14;
inline bool Joint::_internal_has_velocity() const {
  bool value = (_has_bits_[0] & 0x00001000u) != 0;
  return value;
}
inline bool Joint::has_velocity() const {
  return _internal_has_velocity();
}
inline void Joint::clear_velocity() {
  velocity_ = 0;
  _has_bits_[0] &= ~0x00001000u;
}
inline double Joint::_internal_velocity() const {
  return velocity_;
}
inline double Joint::velocity() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.velocity)
  return _internal_velocity();
}
inline void Joint::_internal_set_velocity(double value) {
  _has_bits_[0] |= 0x00001000u;
  velocity_ = value;
}
inline void Joint::set_velocity(double value) {
  _internal_set_velocity(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.velocity)
}

// optional double fudge_factor = 15;
inline bool Joint::_internal_has_fudge_factor() const {
  bool value = (_has_bits_[0] & 0x00002000u) != 0;
  return value;
}
inline bool Joint::has_fudge_factor() const {
  return _internal_has_fudge_factor();
}
inline void Joint::clear_fudge_factor() {
  fudge_factor_ = 0;
  _has_bits_[0] &= ~0x00002000u;
}
inline double Joint::_internal_fudge_factor() const {
  return fudge_factor_;
}
inline double Joint::fudge_factor() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.fudge_factor)
  return _internal_fudge_factor();
}
inline void Joint::_internal_set_fudge_factor(double value) {
  _has_bits_[0] |= 0x00002000u;
  fudge_factor_ = value;
}
inline void Joint::set_fudge_factor(double value) {
  _internal_set_fudge_factor(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.fudge_factor)
}

// optional double limit_cfm = 16;
inline bool Joint::_internal_has_limit_cfm() const {
  bool value = (_has_bits_[0] & 0x00004000u) != 0;
  return value;
}
inline bool Joint::has_limit_cfm() const {
  return _internal_has_limit_cfm();
}
inline void Joint::clear_limit_cfm() {
  limit_cfm_ = 0;
  _has_bits_[0] &= ~0x00004000u;
}
inline double Joint::_internal_limit_cfm() const {
  return limit_cfm_;
}
inline double Joint::limit_cfm() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.limit_cfm)
  return _internal_limit_cfm();
}
inline void Joint::_internal_set_limit_cfm(double value) {
  _has_bits_[0] |= 0x00004000u;
  limit_cfm_ = value;
}
inline void Joint::set_limit_cfm(double value) {
  _internal_set_limit_cfm(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.limit_cfm)
}

// optional double limit_erp = 17;
inline bool Joint::_internal_has_limit_erp() const {
  bool value = (_has_bits_[0] & 0x00008000u) != 0;
  return value;
}
inline bool Joint::has_limit_erp() const {
  return _internal_has_limit_erp();
}
inline void Joint::clear_limit_erp() {
  limit_erp_ = 0;
  _has_bits_[0] &= ~0x00008000u;
}
inline double Joint::_internal_limit_erp() const {
  return limit_erp_;
}
inline double Joint::limit_erp() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.limit_erp)
  return _internal_limit_erp();
}
inline void Joint::_internal_set_limit_erp(double value) {
  _has_bits_[0] |= 0x00008000u;
  limit_erp_ = value;
}
inline void Joint::set_limit_erp(double value) {
  _internal_set_limit_erp(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.limit_erp)
}

// optional double suspension_cfm = 18;
inline bool Joint::_internal_has_suspension_cfm() const {
  bool value = (_has_bits_[0] & 0x00010000u) != 0;
  return value;
}
inline bool Joint::has_suspension_cfm() const {
  return _internal_has_suspension_cfm();
}
inline void Joint::clear_suspension_cfm() {
  suspension_cfm_ = 0;
  _has_bits_[0] &= ~0x00010000u;
}
inline double Joint::_internal_suspension_cfm() const {
  return suspension_cfm_;
}
inline double Joint::suspension_cfm() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.suspension_cfm)
  return _internal_suspension_cfm();
}
inline void Joint::_internal_set_suspension_cfm(double value) {
  _has_bits_[0] |= 0x00010000u;
  suspension_cfm_ = value;
}
inline void Joint::set_suspension_cfm(double value) {
  _internal_set_suspension_cfm(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.suspension_cfm)
}

// optional double suspension_erp = 19;
inline bool Joint::_internal_has_suspension_erp() const {
  bool value = (_has_bits_[0] & 0x00020000u) != 0;
  return value;
}
inline bool Joint::has_suspension_erp() const {
  return _internal_has_suspension_erp();
}
inline void Joint::clear_suspension_erp() {
  suspension_erp_ = 0;
  _has_bits_[0] &= ~0x00020000u;
}
inline double Joint::_internal_suspension_erp() const {
  return suspension_erp_;
}
inline double Joint::suspension_erp() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.suspension_erp)
  return _internal_suspension_erp();
}
inline void Joint::_internal_set_suspension_erp(double value) {
  _has_bits_[0] |= 0x00020000u;
  suspension_erp_ = value;
}
inline void Joint::set_suspension_erp(double value) {
  _internal_set_suspension_erp(value);
  // @@protoc_insertion_point(field_set:gazebo.msgs.Joint.suspension_erp)
}

// optional .gazebo.msgs.Joint.Gearbox gearbox = 20;
inline bool Joint::_internal_has_gearbox() const {
  bool value = (_has_bits_[0] & 0x00000040u) != 0;
  PROTOBUF_ASSUME(!value || gearbox_ != nullptr);
  return value;
}
inline bool Joint::has_gearbox() const {
  return _internal_has_gearbox();
}
inline void Joint::clear_gearbox() {
  if (gearbox_ != nullptr) gearbox_->Clear();
  _has_bits_[0] &= ~0x00000040u;
}
inline const ::gazebo::msgs::Joint_Gearbox& Joint::_internal_gearbox() const {
  const ::gazebo::msgs::Joint_Gearbox* p = gearbox_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Joint_Gearbox&>(
      ::gazebo::msgs::_Joint_Gearbox_default_instance_);
}
inline const ::gazebo::msgs::Joint_Gearbox& Joint::gearbox() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.gearbox)
  return _internal_gearbox();
}
inline void Joint::unsafe_arena_set_allocated_gearbox(
    ::gazebo::msgs::Joint_Gearbox* gearbox) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(gearbox_);
  }
  gearbox_ = gearbox;
  if (gearbox) {
    _has_bits_[0] |= 0x00000040u;
  } else {
    _has_bits_[0] &= ~0x00000040u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Joint.gearbox)
}
inline ::gazebo::msgs::Joint_Gearbox* Joint::release_gearbox() {
  _has_bits_[0] &= ~0x00000040u;
  ::gazebo::msgs::Joint_Gearbox* temp = gearbox_;
  gearbox_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::gazebo::msgs::Joint_Gearbox* Joint::unsafe_arena_release_gearbox() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Joint.gearbox)
  _has_bits_[0] &= ~0x00000040u;
  ::gazebo::msgs::Joint_Gearbox* temp = gearbox_;
  gearbox_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Joint_Gearbox* Joint::_internal_mutable_gearbox() {
  _has_bits_[0] |= 0x00000040u;
  if (gearbox_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Joint_Gearbox>(GetArenaForAllocation());
    gearbox_ = p;
  }
  return gearbox_;
}
inline ::gazebo::msgs::Joint_Gearbox* Joint::mutable_gearbox() {
  ::gazebo::msgs::Joint_Gearbox* _msg = _internal_mutable_gearbox();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.gearbox)
  return _msg;
}
inline void Joint::set_allocated_gearbox(::gazebo::msgs::Joint_Gearbox* gearbox) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete gearbox_;
  }
  if (gearbox) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(gearbox);
    if (message_arena != submessage_arena) {
      gearbox = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, gearbox, submessage_arena);
    }
    _has_bits_[0] |= 0x00000040u;
  } else {
    _has_bits_[0] &= ~0x00000040u;
  }
  gearbox_ = gearbox;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Joint.gearbox)
}

// optional .gazebo.msgs.Joint.Screw screw = 21;
inline bool Joint::_internal_has_screw() const {
  bool value = (_has_bits_[0] & 0x00000080u) != 0;
  PROTOBUF_ASSUME(!value || screw_ != nullptr);
  return value;
}
inline bool Joint::has_screw() const {
  return _internal_has_screw();
}
inline void Joint::clear_screw() {
  if (screw_ != nullptr) screw_->Clear();
  _has_bits_[0] &= ~0x00000080u;
}
inline const ::gazebo::msgs::Joint_Screw& Joint::_internal_screw() const {
  const ::gazebo::msgs::Joint_Screw* p = screw_;
  return p != nullptr ? *p : reinterpret_cast<const ::gazebo::msgs::Joint_Screw&>(
      ::gazebo::msgs::_Joint_Screw_default_instance_);
}
inline const ::gazebo::msgs::Joint_Screw& Joint::screw() const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.screw)
  return _internal_screw();
}
inline void Joint::unsafe_arena_set_allocated_screw(
    ::gazebo::msgs::Joint_Screw* screw) {
  if (GetArenaForAllocation() == nullptr) {
    delete reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(screw_);
  }
  screw_ = screw;
  if (screw) {
    _has_bits_[0] |= 0x00000080u;
  } else {
    _has_bits_[0] &= ~0x00000080u;
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gazebo.msgs.Joint.screw)
}
inline ::gazebo::msgs::Joint_Screw* Joint::release_screw() {
  _has_bits_[0] &= ~0x00000080u;
  ::gazebo::msgs::Joint_Screw* temp = screw_;
  screw_ = nullptr;
#ifdef PROTOBUF_FORCE_COPY_IN_RELEASE
  auto* old =  reinterpret_cast<::PROTOBUF_NAMESPACE_ID::MessageLite*>(temp);
  temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  if (GetArenaForAllocation() == nullptr) { delete old; }
#else  // PROTOBUF_FORCE_COPY_IN_RELEASE
  if (GetArenaForAllocation() != nullptr) {
    temp = ::PROTOBUF_NAMESPACE_ID::internal::DuplicateIfNonNull(temp);
  }
#endif  // !PROTOBUF_FORCE_COPY_IN_RELEASE
  return temp;
}
inline ::gazebo::msgs::Joint_Screw* Joint::unsafe_arena_release_screw() {
  // @@protoc_insertion_point(field_release:gazebo.msgs.Joint.screw)
  _has_bits_[0] &= ~0x00000080u;
  ::gazebo::msgs::Joint_Screw* temp = screw_;
  screw_ = nullptr;
  return temp;
}
inline ::gazebo::msgs::Joint_Screw* Joint::_internal_mutable_screw() {
  _has_bits_[0] |= 0x00000080u;
  if (screw_ == nullptr) {
    auto* p = CreateMaybeMessage<::gazebo::msgs::Joint_Screw>(GetArenaForAllocation());
    screw_ = p;
  }
  return screw_;
}
inline ::gazebo::msgs::Joint_Screw* Joint::mutable_screw() {
  ::gazebo::msgs::Joint_Screw* _msg = _internal_mutable_screw();
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.screw)
  return _msg;
}
inline void Joint::set_allocated_screw(::gazebo::msgs::Joint_Screw* screw) {
  ::PROTOBUF_NAMESPACE_ID::Arena* message_arena = GetArenaForAllocation();
  if (message_arena == nullptr) {
    delete screw_;
  }
  if (screw) {
    ::PROTOBUF_NAMESPACE_ID::Arena* submessage_arena =
        ::PROTOBUF_NAMESPACE_ID::Arena::InternalGetOwningArena(screw);
    if (message_arena != submessage_arena) {
      screw = ::PROTOBUF_NAMESPACE_ID::internal::GetOwnedMessage(
          message_arena, screw, submessage_arena);
    }
    _has_bits_[0] |= 0x00000080u;
  } else {
    _has_bits_[0] &= ~0x00000080u;
  }
  screw_ = screw;
  // @@protoc_insertion_point(field_set_allocated:gazebo.msgs.Joint.screw)
}

// repeated .gazebo.msgs.Sensor sensor = 22;
inline int Joint::_internal_sensor_size() const {
  return sensor_.size();
}
inline int Joint::sensor_size() const {
  return _internal_sensor_size();
}
inline ::gazebo::msgs::Sensor* Joint::mutable_sensor(int index) {
  // @@protoc_insertion_point(field_mutable:gazebo.msgs.Joint.sensor)
  return sensor_.Mutable(index);
}
inline ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Sensor >*
Joint::mutable_sensor() {
  // @@protoc_insertion_point(field_mutable_list:gazebo.msgs.Joint.sensor)
  return &sensor_;
}
inline const ::gazebo::msgs::Sensor& Joint::_internal_sensor(int index) const {
  return sensor_.Get(index);
}
inline const ::gazebo::msgs::Sensor& Joint::sensor(int index) const {
  // @@protoc_insertion_point(field_get:gazebo.msgs.Joint.sensor)
  return _internal_sensor(index);
}
inline ::gazebo::msgs::Sensor* Joint::_internal_add_sensor() {
  return sensor_.Add();
}
inline ::gazebo::msgs::Sensor* Joint::add_sensor() {
  ::gazebo::msgs::Sensor* _add = _internal_add_sensor();
  // @@protoc_insertion_point(field_add:gazebo.msgs.Joint.sensor)
  return _add;
}
inline const ::PROTOBUF_NAMESPACE_ID::RepeatedPtrField< ::gazebo::msgs::Sensor >&
Joint::sensor() const {
  // @@protoc_insertion_point(field_list:gazebo.msgs.Joint.sensor)
  return sensor_;
}

#ifdef __GNUC__
  #pragma GCC diagnostic pop
#endif  // __GNUC__
// -------------------------------------------------------------------

// -------------------------------------------------------------------


typedef boost::shared_ptr<gazebo::msgs::Joint> JointPtr;
// @@protoc_insertion_point(namespace_scope)

}  // namespace msgs
}  // namespace gazebo

PROTOBUF_NAMESPACE_OPEN

template <> struct is_proto_enum< ::gazebo::msgs::Joint_Type> : ::std::true_type {};
template <>
inline const EnumDescriptor* GetEnumDescriptor< ::gazebo::msgs::Joint_Type>() {
  return ::gazebo::msgs::Joint_Type_descriptor();
}

PROTOBUF_NAMESPACE_CLOSE

typedef const boost::shared_ptr<gazebo::msgs::Joint const> ConstJointPtr;
// @@protoc_insertion_point(global_scope)

#include <google/protobuf/port_undef.inc>
#endif  // GOOGLE_PROTOBUF_INCLUDED_GOOGLE_PROTOBUF_INCLUDED_joint_2eproto
