syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface CameraSensor
/// \brief Information about a camera sensor element

import "vector2d.proto";
import "distortion.proto";

message CameraSensor
{
  optional double horizontal_fov = 1;
  optional Vector2d image_size   = 2;
  optional string image_format   = 3;
  optional double near_clip      = 4;
  optional double far_clip       = 5;
  optional bool save_enabled     = 6;
  optional string save_path      = 7;
  optional Distortion distortion = 8;
}
