/* config.h. Generated by CMake for Gazebo. */
#ifndef _GAZEBO_CONFIG_H_
#define _GAZEBO_CONFIG_H_

/* Version number */
#define GAZEBO_MAJOR_VERSION 11
#define GAZEBO_MINOR_VERSION 10
#define GAZEBO_PATCH_VERSION 2

#define GAZEBO_VERSION "11.10"
#define GAZEBO_VERSION_FULL "11.10.2"

#define GAZEBO_VERSION_HEADER "Gazebo multi-robot simulator, version 11.10.2\nCopyright (C) 2012 Open Source Robotics Foundation.\nReleased under the Apache 2 License.\nhttp://gazebosim.org\n\n"

// Warning: the GAZEBO_BUILD_TYPE_* macros are unreliable and deprecated.
// They can change each time you install a new version of gazebo.
// They should not be used and will be removed in the future.
/* #undef GAZEBO_BUILD_TYPE_PROFILE */
/* #undef GAZEBO_BUILD_TYPE_DEBUG */
#define GAZEBO_BUILD_TYPE_RELEASE 1
/* #undef GAZEBO_BUILD_TYPE_COVERAGE */
/* #undef HAVE_OPENAL */
/* #undef HAVE_FFMPEG */
/* #undef HAVE_AVDEVICE */
/* #undef ENABLE_SHADOWS */
/* #undef HAVE_BULLET */
#define HAVE_SIMBODY 1
/* #undef HAVE_DART */
/* #undef HAVE_DART_BULLET */
#define INCLUDE_RTSHADER 1
/* #undef HAVE_GTS */
/* #undef ENABLE_DIAGNOSTICS */
#define HAVE_GDAL 1
#define HAVE_USB 1
/* #undef HAVE_OCULUS */
/* #undef HAVE_SPNAV */
/* #undef HDF5_INSTRUMENT */
/* #undef HAVE_GRAPHVIZ */
#define HAVE_UUID 1
/* #undef USE_EXTERNAL_TINYXML2 */
/* #undef HAVE_OSVR */
/* #undef HAVE_IGNITION_FUEL_TOOLS */

#ifdef GAZEBO_BUILD_TYPE_PROFILE
#include <gperftools/heap-checker.h>
#endif

// ifndef _GAZEBO_CONFIG_H_
#endif
